// Tabkey 0.2
// June 4, 2004
// by Alex Kunin <alx at oxygenworks.net>

window.Tabkey = [0];

function _max(_a, _b) {
	return _a > _b ? _a : _b;
}

function _Tabkey(_definition) {
	this.definition = this._definition = _definition;
	window.Tabkey[this._getInstanceIndex()] = this;
	this._id = '_tabkey_' + this._getInstanceIndex() + '_';
	this._jsPath = 'window.Tabkey[' + this._getInstanceIndex() + ']';
	if (_definition.dynamics) {
		if (_definition.dynamics.init)
			this._dynInit = _definition.dynamics.init;
		if (_definition.dynamics.step)
			this._dynStep = _definition.dynamics.step;
	}
}

$ = _Tabkey.prototype;

$._states = [ 'normal', 'active', 'rollovered' ];

$._getInstanceIndex = function () {
	return this._index = this._index || window.Tabkey.length;
}

$._isFirstItem = function (_index) {
	return _index == 0;
}

$._isLastItem = function (_index) {
	return _index == this._definition.items.length - 1;
}

$._getLayerHtml = function (_id, _content, _additionalAttributes) {
	return '<div style="visibility: hidden; position: absolute; left: 0px; top: 0px;"' + _additionalAttributes+ ' id="' + _id + '">' + _content + '</div>';
}

$._getTemplateForState = function (_state, _index) {
	var _o = null, _actualState = _state;
	while (!_o && _actualState >= 0)
		_o = this._definition[this._states[_actualState--]];
	return this._getLayerHtml(this._getItemLayerId(_state, _index), _o[(this._isFirstItem(_index) && 'first') || (this._isLastItem(_index) && 'last')] || _o['middle'], '');
}

$._getItemLayerId = function (_state, _index) {
	return this._id + _index + '_' + _state;
}

$._getEventLayerId = function (_index) {
	return this._getItemLayerId('ev', _index);
}

$._getLayer = function (_id) {
	return document.getElementById(_id);
}

$._getItemLayer = function (_state, _index) {
	return this._getLayer(this._getItemLayerId(_state, _index));
}

$._getEventLayer = function (_index) {
	return this._getLayer(this._getEventLayerId(_index));
}

$._init = function () {
	var _html = '', _specialFields = {};

	for (var _index in this._definition.items) {
		_specialFields.index = _index;
		for (var _state in this._states) {
			_specialFields.state = this._states._state;
			_html += this._substitute(this._substitute(this._getTemplateForState(_state, _index), this._definition.items[_index]), _specialFields);
			if (this._definition.items[_index].re && document.location.href.match(this._definition.items[_index].re))
				this._activeItemIndex = _index;
		}

		_html += this._getLayerHtml(this._getEventLayerId(_index), '<img src="' + this._definition.blankImage + '" style="width: 100%; height: 100%;" />', this._getHandlers(_index));
	}

	document.write('<div id="' + this._id + '" style="position: relative; width: ' + this._definition.width + 'px; height: ' + this._definition.height + 'px; white-space: nowrap;">' + _html + '</div>');
	window.setTimeout(this._jsPath + '.redraw(STAGE_INIT, ' + this._activeItemIndex + ')', 1);
}

$.onmouseover = function (_index) {
	this._rolloveredItemIndex = _index;
	this._redraw(STAGE_ONMOUSEOVER, _index);
	return true;
}

$.onmouseout = function (_index) {
	this._rolloveredItemIndex = -1;
	this._redraw(STAGE_ONMOUSEOUT, _index);
	return true;
}

$.onclick = function (_index) {
	this._redraw(STAGE_ONCLICK, _index);
	document.location.href = this._definition.items[_index].url || '#';
	return true;
}

$._getHandler = function (_index, _event) {
	return ' on' + _event + '="return ' + this._jsPath + '.on' + _event + '(' + _index + ')"';
}

$._getHandlers = function (_index) {
	return this._getHandler(_index, 'mouseover') + this._getHandler(_index, 'mouseout') + this._getHandler(_index, 'click');
}

var STAGE_INIT = 0, STAGE_ONMOUSEOVER = 1, STAGE_ONMOUSEOUT = 2, STAGE_ONCLICK = 3;

$._dynInit = function (_items, _stage, _itemIndex) {
}

$._dynStep = function (_items) {
	var _left = 0;

	for (var i in _items) {
		_items[i].left = _left;
		_left += _items[i].width + this.definition.spacing;
	}

	return 0;
}

$.__redraw = function () {
	var _delay = this._dynStep(this._temporaryItems);

	for (var _index in this._definition.items) {
		var _visibleState = this._rolloveredItemIndex == _index ? 2 : (this._activeItemIndex == _index ? 1 : 0);

		for (var _state in this._states) {
			var _layer = this._getItemLayer(_state, _index);
			_layer.style.left = this._temporaryItems[_index].left + 'px';
			_layer.style.top = this._temporaryItems[_index].top + 'px';
			_layer.style.visibility = _state == _visibleState ? 'visible' : 'hidden';
		}

		var _layer = this._getEventLayer(_index);
		_layer.style.left = this._temporaryItems[_index].left + 'px';
		_layer.style.top = this._temporaryItems[_index].top + 'px';
		_layer.style.width = this._temporaryItems[_index].width + 'px';
		_layer.style.height = this._temporaryItems[_index].height + 'px';
		_layer.style.visibility = 'visible';
	}

	if (_delay)
		window.setTimeout(this._jsPath + '.__redraw()', _delay)
}

$.redraw = $._redraw = function (_stage, _itemIndex) {
	if (!this._temporaryItems) {
		this._temporaryItems = [];

		for (var _index in this._definition.items) {
			var _layer = this._getItemLayer(0, _index);

			this._temporaryItems[this._temporaryItems.length] = {
				left:_layer.offsetLeft,
				top:_layer.offsetTop,
				width:_layer.offsetWidth,
				height:_layer.offsetHeight,
				isRollovered:this._rolloveredItemIndex == _index,
				isActive:this._activeItemIndex == _index
			};
		}
	}

	this._dynInit(this._temporaryItems, _stage, _itemIndex);
	this.__redraw();
}

$._substitute = function (_template, _fields) {
	for (var _field in _fields)
		_template = _template.replace('{' + _field + '}', _fields[_field]);

	return _template;
}

function GenerateTabkey(_definition) {
	(new _Tabkey(_definition))._init();
}
